unit BackgroundUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
	TBackgroundForm = class(TForm)
		procedure FormShow(Sender: TObject);
		procedure AppException(Sender: TObject; E: Exception);
	private
	public
	end;

var
	BackgroundForm: TBackgroundForm;

implementation

{$R *.DFM}

uses
	Registry, ShlObj, ActiveX,
	UtilsDos, UtilsWin,
	WizardUnit, InstallUnit;

procedure TBackgroundForm.FormShow(Sender: TObject);
procedure Del(const FileName: string);
begin
	if FileExists (WithBackslash (InstallDir) + FileName) then try
		DeleteFile (WithBackslash (InstallDir) + FileName);
	except end;
end;
procedure DelDir(const DirName: string);
begin
	if DirExists (WithBackslash (InstallDir) + DirName) then try
		DelTree (WithBackslash (InstallDir) + DirName);
	except end;
end;
procedure DelLink(const Link: string);
begin
	if Length (ProgGroupPath) > 0 then begin
		if FileExists (WithBackslash (ProgGroupPath) + Link + '.lnk') then try
			DeleteFile (WithBackslash (ProgGroupPath) + Link + '.lnk');
		except end else if FileExists (WithBackslash (ProgGroupPath) + Link + '.pif') then try
			DeleteFile (WithBackslash (ProgGroupPath) + Link + '.pif');
		except end;
	end;
end;
procedure CreateLink(const Link, FileName: string; const Arguments: string = ''; const WorkDir: string = '');
var
	ShellLink: IShellLink;
	PersistFile: IPersistFile;
	WidePath: array [0..MAX_PATH] of Word;
begin
	if (Length (ProgGroupPath) > 0) and (CoCreateInstance (CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER, IShellLink, ShellLink) = NOERROR) and (ShellLink.QueryInterface (IPersistFile, PersistFile) = NOERROR) then begin
		ShellLink.SetPath (PChar (FileName));
		ShellLink.SetArguments (PChar (Arguments));
		ShellLink.SetWorkingDirectory (PChar (WorkDir));
		ShellLink.SetDescription (PChar (Link));
		MultiByteToWideChar (CP_ACP, 0, PChar (WithBackslash (ProgGroupPath) + Link + '.lnk'), -1, @WidePath, MAX_PATH);
		PersistFile.Save (@WidePath, True);
	end;
end;
var
	L: TStringList;
	I,
	P: Integer;
	Modified: Boolean;
	RegStr: string;
begin
	Application.OnException := AppException;
	with TWizardForm.Create (Self) do try
		if ShowModal = mrOK then begin
			Del ('Readme.txt');
			Del ('License.txt');
			Del ('Doc\tigcclib.chm');
			Del ('Bin\cpp.exe');
			Del ('Bin\cpp0.exe');
			Del ('Bin\link.exe');
			Del ('Bin\strip.exe');
			Del ('Bin\tovar.exe');
			Del ('Bin\ar.exe');
			Del ('Bin\objcopy.exe');
			Del ('Bin\cygwin1.dll');
			Del ('Lib\tipatch.lib');
			Del ('Lib\tipatchmain.lib');
			Del ('Lib\pstarter-titanium.o');
			Del ('Include\C\pstarter.s');
			Del ('Include\C\pstarter.o');
			Del ('Include\C\tipatch.lib');
			Del ('Include\C\tipatchmain.lib');
			Del ('Include\C\Sources');
			if not InstallExamplesCheckBox.Checked then
				DelDir (ExamplesLocation);
			if not InstallTIGCCCheckBox.Checked then
				Del (XaviersTIGCCLocation);
			if not InstallIDECheckBox.Checked then
				Del (IDELocation);
			if not InstallA68kCheckBox.Checked then begin
				Del (A68kLocation);
				DelDir (A68kIncludeLocation);
			end;
			if not InstallPackCheckBox.Checked then begin
				Del (PackLocation);
				Del (PStarterLocation);
			end;
			with TInstallForm.Create (Self) do try
				Aliases.Values ['INSTALLDIR'] := InstallDir;
				Aliases.Values ['WINDIR'] := WinDir;
				Aliases.Values ['WINDOWSDIR'] := WinDir;
				Aliases.Values ['SYSDIR'] := SysDir;
				Aliases.Values ['SYSTEMDIR'] := SysDir;
				InstallIDE := InstallIDECheckBox.Checked;
				InstallTIGCC := InstallTIGCCCheckBox.Checked;
				InstallExamples := InstallExamplesCheckBox.Checked;
				InstallA68k := InstallA68kCheckBox.Checked;
				InstallPack := InstallPackCheckBox.Checked;
				NoSystemFolderUse := NoSystemFolderUseCheckBox.Checked;
				if ShowModal = mrOK then begin
					if not DirExists (WithBackslash (InstallDir) + ProjectsLocation) then try
						MkDir (WithBackslash (InstallDir) + ProjectsLocation);
					except end;
					Modified := False;
					L := TStringList.Create;
					try
						if FileExists ('C:\AUTOEXEC.BAT') then
							L.LoadFromFile ('C:\AUTOEXEC.BAT');
					except end;
					for I := L.Count - 1 downto 0 do
						if L [I] = EnvRem then begin
							Modified := True;
							L.Delete (I);
							if I < L.Count then
								L.Delete (I);
						end;
					if InstallTIGCCCheckBox.Checked and (not SystemNT) and PathLineCheckBox.Checked then begin
						Modified := True;
						L.Add (EnvRem);
						L.Add ('SET PATH=%PATH%;' + DosFold);
					end;
					if Modified then
						try
							L.SaveToFile ('C:\AUTOEXEC.BAT');
						except end;
					L.Free;
					with TRegistry.Create do try
						try
							RootKey := HKey_Local_Machine;
							if OpenKey ('\Software\TIGCC Team\TIGCC', True) then begin
								WriteString ('Program Folder', InstallDir);
								if (not ValueExists ('Program Group')) or (ReadString ('Program Group') <> ProgramGroupBox.Text) then
									WriteBool ('Program Group Existed', ProgramGroupBox.Items.IndexOf (ProgramGroupBox.Text) >= 0);
								WriteString ('Program Group', ProgramGroupBox.Text);
								if not NoSystemFolderUse then begin
									OpenKey ('\Software\Microsoft\Windows\CurrentVersion\Uninstall\TI-GCC', True);
									WriteString ('DisplayName', 'TIGCC Calculator SDK');
									WriteString ('UninstallString', '"' + WithBackslash (SysDir) + 'TIGCCUninst.exe"');
								end;
								RootKey := HKey_Classes_Root;
								OpenKey ('\.tpr', True);
								WriteString ('', 'TIGCCProject');
								OpenKey ('\TIGCCProject', True);
								WriteString ('', 'TIGCC Project');
								OpenKey ('DefaultIcon', True);
								WriteString ('', WithBackslash (InstallDir) + IDELocation + ',0');
								OpenKey ('\TIGCCProject\Shell\open\command', True);
								WriteString ('', '"' + WithBackslash (InstallDir) + IDELocation + '" "%1"');
								if SystemNT then try
									RegStr := '';
									RootKey := HKey_Local_Machine;
									if OpenKey ('\System\CurrentControlSet\Control\Session Manager\Environment', True) then begin
										if ValueExists ('Path') then try
											RegStr := ReadString ('Path');
										except end;
										P := Pos (UpperCase (DosFold), UpperCase (RegStr));
										if P > 0 then begin
											if P > 1 then
												Dec (P);
											Delete (RegStr, P, Length (DosFold) + 1);
										end;
										if InstallTIGCCCheckBox.Checked and RegPathLineCheckBox.Checked then begin
											if Length (RegStr) > 0 then
												RegStr := RegStr + ';';
											RegStr := RegStr + DosFold;
										end;
										WriteExpandString ('Path', RegStr);
									end else
										Abort;
								except
									ShowDefaultMessageBox ('Could not set up path variable for command-line compiler. You probably do not have the required access rights.', 'Error', mtProgramError);
								end;
							end else
								Abort;
						except
							ShowDefaultMessageBox ('Could not set up registry values. You probably do not have the required access rights.', 'Error', mtProgramError);
						end;
					finally
						Free;
					end;
					if ProgPath <> '' then begin
						ProgGroupPath := WithBackslash (ProgPath) + ProgramGroupBox.Text;
						if not DirExists (ProgGroupPath) then try
							MkDir (ProgGroupPath);
						except end;
					end;
					CoInitialize (nil);
					if InstallIDE then
						CreateLink ('TIGCC IDE', WithBackslash (InstallDir) + IDELocation)
					else
						DelLink ('TIGCC IDE');
					if InstallTIGCC then
						CreateLink ('TIGCC Command Line Prompt', 'Command.com', '', DosFold)
					else
						DelLink ('TIGCC Command Line Prompt');
					CreateLink ('TIGCC Documentation', WithBackslash (InstallDir) + HelpLocation);
					if not NoSystemFolderUse then
						CreateLink ('Uninstall TIGCC', WithBackslash (SysDir) + 'TIGCCUninst.exe');
					if InstallIDE then begin
						if Application.MessageBox
							('The program has been installed successfully.  Do you want to run it now?',
							'Installation Successful',
							mb_YesNo or mb_IconQuestion) = idYes then
							WinExec (PChar ('"' + WithBackslash (InstallDir) + IDELocation + '"'), sw_Show);
					end else
						Application.MessageBox
							('The program has been installed successfully.',
							'Installation Successful',
							mb_OK or mb_IconInformation);
				end;
			finally
				Free;
			end;
		end;
	finally
		Free;
	end;
	Close;
end;

procedure TBackgroundForm.AppException(Sender: TObject; E: Exception);
begin
	if E is EInOutError then
		ShowDefaultMessageBox (E.Message, 'Error', mtProgramError)
	else
		ShowDefaultMessageBox ('Internal Error: Exception ' + E.ClassName + #13#10#13#10'Please fill out a bug report form at http://tigcc.ticalc.org/.', 'Internal Error', mtProgramError);
end;

end.
